import numpy as np
import matplotlib.pyplot as plt
from scipy.integrate import ode

plt.rcParams['font.size'] = 16
#--- geometry ---
L = 250e-6 
w = 5e-6
h = 1e-6
d = 1e-6
#--- material constants ---
rho = 2330
Y = 150e9
eps0 = 8.854e-12
#--- system parameters ---
m = w*h*L/3*rho
k = .25*Y*w*h**3/(L**3)
Q = 1000
b = np.sqrt(k*m)/Q
Co = eps0*w*L/d
R = 1e15
alpha =.5
beta = .5
Vo = 3
N = Vo*eps0*w*L/(d**2)/2
# --- integration parameters ---
wo = np.sqrt(k/m)
tEnd = 100*2*np.pi/wo
dt = tEnd/1e5
time = []

# --- ode model
def oscModel( t, x):
    vout = beta*np.tanh( alpha*(x[1]-Vo) )+Vo
    dxdt = np.dot( np.array([[-b/m, N**2/m, -N**2/m],
                             [1/Co,-1/R/Co,0],
                             [k*N**2,0,0]]),x )+ np.array([0, vout/R/Co,0])
    return( dxdt )
#--- transfer function ---
def H(w):
    out = ( (m*(1j*w)**2 + b*(1j*w) + 1/k) / 
            (m*R*Co*(1j*w)**3+(b*R*Co+m)*(1j*w)**2+(R*Co*k+b)*(1j*w)+R*N**2+1/k) )
    return(out)
# --- solveODE
def solveODE( ):
    v = []
    v0 = [0, 2.5,2.7]
    r = ode(oscModel).set_integrator('zvode',method = 'bdf')
    r.set_initial_value(v0,0)
    
    while r.successful() and r.t<tEnd:
        r.integrate(r.t+dt)
        time.append(r.t)
        v.append(r.y)
    v = np.array(v)
    plt.figure()
    plt.axes((.15,.15,.8,.8))
    plt.plot(time,v[:,1])
    plt.xlabel('Time $t$ [sec]')
    plt.grid(True)
    plt.ylabel('Output voltage $v_{out}(t)$ [V]')
    plt.show()
# --- define ode problem 
def plotMagResp( ):
    w = np.linspace(.9*wo, 1.1*wo,3000)
    Ho = H(w)
    plt.figure()
    plt.axes((.15,.15,.75,.75))
    plt.plot(w/2/np.pi/1e3,abs(Ho), color = 'k', linewidth = 1.5)
    yl = plt.gca().get_ylim()
    plt.plot(wo/2/np.pi/1e3*np.ones(2), yl,linestyle = '--',color = 'gray')
    plt.grid(True)
    plt.xlabel('Frequency $f$ [kHz]')
    plt.ylabel('Magnitude response')
    plt.show()

plotMagResp()
solveODE( )