import numpy as np                                                           
import matplotlib.pyplot as plt                                              
from scipy.integrate import ode                                              
                                                                             
plt.rcParams['font.size'] = 16                                               
#--- geometry ---                                                            
L = 200e-6    # electrode length                                                               
w = 10e-6     # electrode width                                                                
h = 2e-6    # electrode height                                                                
d = 2e-6     # electrode spacing
#--- material constants ---                                                  
rho = 2330        # mass density                                                               
Y = 150e9         # Young modulus                                                                
eps0 = 8.854e-12  # dielectric constant                                                           
#--- system parameters ---                                                   
m = n*w*h*L*rho+2*(n-1)*(w+d)*w*h*rho                                                              
k = 2*Y*h*(w/Ls)**3                                                      
Q = 10000.                                                                     
b = np.sqrt(k*m)/Q       
Vo = 15.0 
N = Vo*eps0*wL/d**2 
print '\nMechanical system:'                                        
print '\t m = '+('%.3e')%(m)+ ' [kg]'
print '\t k = '+('%.3e')%(k)+ ' [N/m]'
print '\t b = '+('%.3e')%(b)+ ' [Ns/m]'
print '\t f = '+('%.3e')%(np.sqrt(k/m)/2/np.pi/1e6)+ ' [MHz]'
print '-'*50
#--- ZMEMS components ----
Co = 2*n*eps0*h*L/d; #Co = 1e-12                                                       
Leq = m/N**2; #Leq = 100e-3
Req = b/N**2; #Req = 1e3
Ceq = N**2/k; #Ceq = Co/5;
print '\nComponents of the reflected impedance:'
print '\t  Co = '+('%.3e')%(Co)+ ' [F]'
print '\t  N  = '+('%.3e')%(N)+ ' [C/m]'
print '\t Leq = '+('%.3e')%(Leq)+ ' [H]'
print '\t Ceq = '+('%.3e')%(Ceq)+ ' [F]'
print '\t Req = '+('%.3e')%(Req)+ ' [Omega]'
print '\t  fo = '+('%.3e')%(1/np.sqrt(Leq*Ceq)/2/np.pi/1e6) + ' [MHz]'
print '\t  Df = '+('%.3e')%(Ceq/Co*1/np.sqrt(Leq*Ceq)/2/np.pi) + ' [Hz]'
print '-'*50

# Co = Ceq

# --- CKT components ---
R = 100                                                                     
C1 = Co                                                                    
alpha = 5                                                                    
beta = 1                                                                                                                                       
Cp = Co*Ceq/(Co + Ceq)                                           
# --- integration parameters ---                                             
wo = np.sqrt(k/m)                                                            
tEnd = 20*2*np.pi/wo                                                        
dt = tEnd/1e3                                                                
time = []                                                                    
xDC0 = np.array([0, Vo, Vo])                                                           
xDC1 = np.array([0, Vo*C1/(Cp+C1), Vo*C1/(Cp+C1), Vo*Cp/(Cp+C1)])                  
# --- ode models------------------------------------------------------------                                                              
def xDot0( t, x):  
    """
    this is a serial resistance model
    """                                                         
    vout = beta*np.tanh( alpha*(x[1]-Vo ))+Vo                     
    A = np.array([[-Req/Leq, 1/Leq, -1/Leq],                                 
                  [-1/Co,-1/R/Co,0],                                  
                  [1/Ceq,0,0]])                                    
    dxdt = np.dot(A,x )+ np.array([0, (vout)/R/Co,0])               
    return( dxdt )

def xDot1( t, x):
    """
    this is a serial capacitance + resistance model
    """                                                         
    vout = beta*np.tanh( alpha*(x[1]-C1*Vo/(C1+Cp) ))+Vo
    A = np.array([[-Req/Leq, 1/Leq, -1/Leq,0],
                  [1/Co,-1/R/Co,0,-1/R/Co],
                  [1/Ceq,0,0,0],
                  [0,-1/R/C1,0,-1/R/C1]])
#     print A
    dxdt = np.dot(A,x )+ np.array([0, vout/R/Co,0, vout/R/C1])
    return( dxdt )                                                             
#--- transfer function ---                                                   
def getZMEMS(w):                                                                    
    ZMEMS = 1/(1j*w*Co+1/(Leq*(1j*w) + Req+1/(1j*w*Ceq)))
    return( ZMEMS )                                                              
# --- solveODE                                                               
def solveODE( ):                                                             
    v = []                                                                   
    x0 = xDC0-np.array([1e-5,0,0])                    
#     x0 = xDC1-np.array([1.e-5,0,0,0])
    r = ode(xDot0).set_integrator('zvode',method = 'bdf')                 
    r.set_initial_value(x0,0)                                                
    while r.successful() and r.t<tEnd:                                       
        r.integrate(r.t+dt)                                                  
        time.append(r.t)                                                     
        v.append(r.y)                                                        
    v = np.array(v)                                                          
    plt.figure()                                                             
    plt.axes((.15,.15,.8,.8))       
    plt.plot(time,v[:,1],time,beta*np.tanh( alpha*(v[:,1]-Vo ))+Vo)                                                    
    plt.xlabel('Time $t$ [sec]')                                             
    plt.grid(True)                                                           
    plt.ylabel('Voltage [V]')                            
    plt.legend((r'$v_{C_o} (t)$',r'$v_{out} (t)$',r'$v_k (t)$'))
    plt.show()                                                               
# --- define ode problem      
def pltAnotats(w,Z):
    M = max(np.real(Z))
    plt.gca().annotate(r'$|H|_{max}$', xy=(wo/np.pi/2/1e3, M),  xycoords='data',
                xytext=(-70, 10), textcoords='offset points',
                arrowprops=dict(arrowstyle="->",connectionstyle="arc,angleA=0,armA=30,rad=10")
                )
    ind0, = np.where(np.real(Z) == M)
    w1 = np.interp(M/np.sqrt(2),np.real(Z[:ind0]), w[:ind0])
    plt.gca().annotate(r'$\frac{|H|_{max}}{\sqrt{2}}$', xy=(w1/np.pi/2/1e3, M/np.sqrt(2)),  xycoords='data',
                xytext=(-50, 30), textcoords='offset points',
                arrowprops=dict(arrowstyle="->")
                )
    w2 = np.interp(M/np.sqrt(2),np.real(Z[-1:ind0:-1]), w[-1:ind0:-1])
    plt.gca().annotate(r'$\frac{|H|_{max}}{\sqrt{2}}$', xy=(w2/np.pi/2/1e3, M/np.sqrt(2)),  xycoords='data',
                xytext=(20, 30), textcoords='offset points',
                arrowprops=dict(arrowstyle="->")
                )
    plt.text((w1+w2)/2/2/np.pi/1e3,M/np.sqrt(2),(r'$Q =\frac{\Delta\omega}{\omega_o} =$%d')%(wo/(w2-w1)),horizontalalignment = 'center')
    plt.plot(w1/2/np.pi/1e3*np.ones(2), (yl[0],M/np.sqrt(2)),linestyle = '--',color = 'gray')
    plt.plot(w2/2/np.pi/1e3*np.ones(2), (yl[0],M/np.sqrt(2)),linestyle = '--',color = 'gray')     
def plotZ( ):  
    wo = 1/np.sqrt(Leq*Ceq)                                                        
    w = np.linspace(.9*wo, 1.1*wo,3000)                                      
    Z = getZMEMS(w)                                                                
    plt.figure()                                                             
    ax1= plt.axes((.15,.15,.7,.75))                                              
    plt.plot(w/2/np.pi/1e3,np.real(Z), color = 'k', linewidth = 1.5)            
    yl = plt.gca().get_ylim()                                                
    plt.plot(2/np.pi/1e3*np.ones(2), yl,linestyle = '--',color = 'gray')  
    plt.grid(True)                                                           
    plt.xlabel(r'Frequency $\omega/(2\pi)$ [kHz]')                                        
    plt.ylabel(r'$\Re \{ Z_{MEMS} \}$')   
    ax2 = plt.twinx()
    plt.plot(w/2/np.pi/1e3,np.imag(Z), color = 'gray', linewidth = 1.5)   
    plt.ylabel('$\Im \{ Z_{MEMS} \}$')                                   
    plt.show()                                                               
                                                                             
plotZ()                                                              
# solveODE( )                                                                