import numpy as np                                                           
import matplotlib.pyplot as plt                                              
from scipy.integrate import ode                                              
                                                                             
plt.rcParams['font.size'] = 9
plt.rcParams['font.family'] = 'arial'  
bgClr = 0.85*np.ones(3)                                            
#--- Input parameters: geometry, Vbltage and quality factor --- 
ra = 200e-6    # radius of the plate                                                              
h = 2e-6    # electrode height                                                                
d = 2e-6     # electrode spacing
Vb = 7     # bias Vbltage
Q  = 200     # quality factor
print '\nInput parameters: geometry, Vbias, and Q'
print '\tra = '+('%.3e')%(ra)+ ' [m]   plate radius)'  
print '\t h = '+('%.3e')%(h)+ ' [m]   (electrode thickness)'
print '\t d = '+('%.3e')%(d)+ ' [m]   (electrode spacing)'
print '\tVb = '+('%.3e')%(Vb)+ ' [V]   (bias voltage)'
print '\t Q = '+('%.3e')%(Q)+  '       (quality factor)'
print '-'*50  
#--- material constants ---                                                  
rho = 2330        # mass density                                                               
Y = 150e9         # Young modulus      
nu = .3           # Poisson ratio                                       
eps0 = 8.854e-12  # dielectric constant                                                           
#--- system parameters ---                                                   
m = 0.81*ra**2*np.pi*h*rho                                                             
k = 16./3*Y*h**3/(1-nu)*np.pi/(ra**2)-eps0*ra**2*np.pi/d**3*Vb # Eq. 6.97                                                                                                                 
b = np.sqrt(k*m)/Q       
Fo = 0.5*eps0*ra**2*np.pi/d**2*Vb**2
xo = Fo/k
N = Vb*eps0*ra**2*np.pi/d**2 
print '\nMechanical system:'                                        
print '\t m = '+('%.3e')%(m)+ ' [kg]'
print '\t k = '+('%.3e')%(k)+ ' [N/m]'
print '\t b = '+('%.3e')%(b)+ ' [Ns/m]'
print '\tFo = '+('%.3e')%(Fo)+' [N]'
print '\txo = '+('%.3e')%(xo)+' [m]'
print '\t f = '+('%.3e')%(np.sqrt(k/m)/2/np.pi)+ ' [Hz]'
print '-'*50
#--- ZMEMS components ----
Co = eps0*ra**2*np.pi/d                                                        
Leq = m/N**2; 
Req = b/N**2; 
Ceq = N**2/k; 
fo = 1/np.sqrt(Leq*Ceq)/2/np.pi
Df = Ceq/Co*1/np.sqrt(Leq*Ceq)/2/np.pi/2
print '\nComponents of the reflected impedance:'
print '\t  Co = '+('%.3e')%(Co)+ ' [F]'
print '\t  N  = '+('%.3e')%(N)+ ' [C/m]'
print '\t Leq = '+('%.3e')%(Leq)+ ' [H]'
print '\t Ceq = '+('%.3e')%(Ceq)+ ' [F]'
print '\t Req = '+('%.3e')%(Req)+ ' [Ohm]'
print '\t  fo = '+('%.3e')%(fo) + ' [Hz]'
print '\t  Df = '+('%.3e')%(Df) + ' [Hz]'
print '-'*50
# --- CKT components ---
R = 3e6                                                                     
C1 = Co                                                                    
alpha = 5                                                                    
beta = 1.2                                                                                                                                       
Cp = Co*Ceq/(Co + Ceq)                                           
# --- integration parameters ---                                             
wo = np.sqrt(k/m)                                                            
tEnd = 100*2*np.pi/wo                                                        
dt = tEnd/1e5                                                                
time = []                                                                    
xDC0 = np.array([0, Vb, Vb])                                                           
xDC1 = np.array([0, Vb*C1/(Cp+C1), Vb*C1/(Cp+C1), Vb*Cp/(Cp+C1)])                  
# --- ode models------------------------------------------------------------                                                              
def xDot0( t, x):  
    """
    this is a serial resistance model
    """                                                         
    Vbut = beta*np.tanh( alpha*(x[1]-Vb ))+Vb  
    Vbut = Vb*np.tanh( alpha*(x[1]-Vb )/Vb)+Vb  
#     alpha = 1e5   
#     Vbut = Vb*np.tanh( alpha*(x[1]-Vb)/Vb)+Vb              
    A = np.array([[-Req/Leq, 1/Leq, -1/Leq],                                 
                  [-1/Co,-1/R/Co,0],                                  
                  [1/Ceq,0,0]])                                    
    dxdt = np.dot(A,x )+ np.array([0, (Vbut)/R/Co,0])               
    return( dxdt )
#--- transfer function ---                                                   
def getZMEMS(w):                                                                    
    ZMEMS = 1/(1j*w*Co+1/(Leq*(1j*w) + Req+1/(1j*w*Ceq)))
    return( ZMEMS )                                                              
# --- solveODE                                                               
def solveODE( ):                                                             
    v = []                                                                   
    x0 = xDC0+np.array([1e-6,0,0])   
    x0 = xDC0+np.array([-6.5e-7,0,0])                 
#     x0 = xDC1-np.array([1.e-5,0,0,0])
    r = ode(xDot0).set_integrator('zVbde',method = 'bdf')                 
    r.set_initial_value(x0,0)  
    time = []                                              
    while r.successful() and r.t<tEnd:                                       
        r.integrate(r.t+dt)                                                  
        time.append(r.t)                                                     
        v.append(r.y)                                                        
    v = np.array(v)                                                          
    plt.figure()                                                             
    ax1= plt.axes((.15,.05,.8,.4)) 
    time = np.array(time)      
    plt.plot(time*1e6,v[:,1],color ='k')
    plt.ylabel('Filtered voltage [V]')
    plt.grid(True)
    ax2 = plt.axes((.15,.55,.8,.4),sharex=ax1)
    plt.plot(time*1e6,beta*np.tanh( alpha*(v[:,1]-Vb ))+Vb,color = 'k')                                                    
    plt.xlabel(r'Time $t$ [$\mu$s]')                                             
    plt.grid(True)                                                           
    plt.ylabel('Amplifier Output [V]')                            
    plt.show()                                                               
     
def plotZ( ):  
    wo = 1/np.sqrt(Leq*Ceq)   
    wRange = (.95*wo, 1.05*wo)                                                     
    w = np.linspace(wRange[0], wRange[1],3000)                                      
    Z = getZMEMS(w)                                                               
    plt.figure(facecolor = bgClr)                      
    # mechanical transfer function                                        
    ax1= plt.axes((.15,.72,.7,.25))
    plt.plot(w/2/np.pi/1e3,1/np.sqrt((1-(w/wo)**2)**2+(w/wo/Q)**2),color = 'k',
    linewidth = 1.5)
    yl = plt.gca().get_ylim()                                                
    plt.plot(wo/2/np.pi/1e3*np.ones(2), yl,linestyle = '--',color = 'gray')  
#     plt.grid(True)
    plt.ylabel(r'$|H_{x/f}(\omega)|/(1/k)$')
    plt.gca().annotate(r'$\frac{1}{2\pi} \sqrt{\frac{k_{eff}}{m_{eff}}}$',xy=(fo/1e3,0.1),xytext=(-140, 40), textcoords='offset points',
                       arrowprops=dict(arrowstyle="->",connectionstyle="arc3,rad=-.5"),fontsize = 9)       #connectionstyle = "angleA=0,armA=30,rad=10"
    plt.text(w[-1]/2/np.pi/1e3,yl[1]*.95,'  (a)',horizontalalignment = 'left',verticalalignment = 'top')

    # Re{Z_MEMS} 
    ax2 = plt.axes((.15,.42,.7,.25),sharex = ax1)
    plt.plot(w/2/np.pi/1e3,np.real(Z)/1e6, color = 'k', linewidth = 1.5)            
    yl = plt.gca().get_ylim()                                                
    plt.plot(wo/2/np.pi/1e3*np.ones(2), yl,linestyle = '--',color = 'gray')  
    plt.plot((fo+Df)/1e3*np.ones(2),yl, linestyle = '--',color = 'gray')
    plt.ylabel(r'Re ($Z\underbar _{MEMS} $) [M$\Omega$]')
    plt.text(w[-1]/2/np.pi/1e3,yl[1]*.95,'  (b)',horizontalalignment = 'left',verticalalignment = 'top')
#     plt.gca().annotate(r'$\frac{1}{2\pi \sqrt{L_{eq}C_{eq}}}$',xy=(fo/1e3,0.1),xytext=(-100, 30), textcoords='offset points',
#                        arrowprops=dict(arrowstyle="->",connectionstyle="arc3,rad=-.5"),fontsize = 24)      
#     plt.gca().annotate(r'$\omega_A$',xy=(fo/1e3+Df/1e3,0.1),xytext=(40, 30), textcoords='offset points',
#                        arrowprops=dict(arrowstyle="->",connectionstyle="arc3,rad=.5"),fontsize = 18)       
    M = max(np.real(Z))/1e6
#     plt.text((fo+Df/2)/1e3,M/2,r'$\frac{\Delta \omega}{2\pi}$',
#              horizontalalignment = 'center',verticalalignment = 'bottom',
#              fontsize = 24)
#     plt.gca().annotate('',xy = (fo/1e3,M/2),xytext = ((fo+Df)/1e3,M/2),arrowprops=dict(arrowstyle = "<->"))
    # Im{Z_MEMS}
    ax3 = plt.axes((.15,.12,.7,.25),sharex = ax1)
    plt.plot(w/2/np.pi/1e3,np.imag(Z)/1e6, color = 'k', linewidth = 1.5) 
    yl = plt.gca().get_ylim()
    plt.plot(wo/2/np.pi/1e3*np.ones(2),yl,color='gray',linestyle = '--');  
    plt.plot((fo+Df)/1e3*np.ones(2),yl, linestyle = '--',color = 'gray')
    plt.ylabel(r'Im$(Z\underbar _{MEMS}$) [M$\Omega$]')
    plt.xlabel(r'Frequency $\omega/(2\pi)$ [kHz]')                                          
    plt.gca().set_xlim( np.array(wRange)/2/np.pi/1e3 )             
    ind, = np.where(w<=3*wo)
    plt.fill( np.concatenate((w[ind]/2/np.pi/1e3,w[ind[-1:0:-1]]/2/np.pi/1e3)),
              np.concatenate((np.imag(Z[ind]/1e6),np.zeros(len(ind)-1))),
              facecolor = np.ones(3)*.8,label = '"$C$"')
#     ax3.annotate(r'$\frac{1}{2\pi \sqrt{L_{eq}C_{eq}}}$',xy=(fo/1e3,0),xytext=(-100, -30), textcoords='offset points',
#                        arrowprops=dict(arrowstyle="->",connectionstyle="arc3,rad=.5"),fontsize = 24)       #connectionstyle = "angleA=0,armA=30,rad=10"
#     ind1, = np.where(w<=wo+2*np.pi*Df)
#     indA = np.arange(max(ind),max(ind1))
#     plt.fill( np.concatenate((w[indA]/2/np.pi/1e3,w[indA[-1:0:-1]]/2/np.pi/1e3)),
#               np.concatenate((np.imag(Z[indA]/1e6),np.zeros(len(indA)-1))),
#               hatch = '|',facecolor = bgClr,label = '"$L$"')
#     ax3.annotate(r'$\frac{1}{2\pi \sqrt{L_{eq}C_{eq}}}$',xy=(fo/1e3,0),xytext=(-100, -30), textcoords='offset points',
#                        arrowprops=dict(arrowstyle="->",connectionstyle="arc3,rad=.5"),fontsize = 24)       #connectionstyle = "angleA=0,armA=30,rad=10"
# #     ind2, = np.where(w<=wo+2*np.pi*Df)
#     indB = np.arange(max(ind1),len(w))
#     plt.fill( np.concatenate((w[indB]/2/np.pi/1e3,w[indB[-1:0:-1]]/2/np.pi/1e3)),
#               np.concatenate((np.imag(Z[indB]/1e6),np.zeros(len(indB)-1))),
#               hatch = '/',facecolor = bgClr)
#     ax3.annotate(r'$\frac{1}{2\pi \sqrt{L_{eq}C_{eq}}}$',xy=(fo/1e3,0),xytext=(-100, -30), textcoords='offset points',
#                        arrowprops=dict(arrowstyle="->",connectionstyle="arc3,rad=.5"),fontsize = 24)       #connectionstyle = "angleA=0,armA=30,rad=10"
#     plt.gca().annotate(r'$\omega_A$',xy=(fo/1e3+Df/1e3,0.1),xytext=(20, 15), textcoords='offset points',
#                        arrowprops=dict(arrowstyle="->",connectionstyle="arc3,rad=.5"),fontsize = 18)       
#     a= plt.legend(loc = 'lower left',bbox_to_anchor = (.9,-.3))
#     a.get_frame().set_facecolor(bgClr)
    plt.text(w[-1]/2/np.pi/1e3,yl[1]*.95,'  (c)',horizontalalignment = 'left',verticalalignment = 'top')
    plt.show()   
    plt.gcf().set_figheight(4)
    plt.gcf().set_figwidth(6.5)                                                            
    plt.savefig('Fig_5p52.eps')         
                                                                             
plotZ()                                                              
# solveODE( )                                                                